
// Code for the Fab Academy Final project code. AUtomated water fountain for cat.
// create by: Francis Lance
//---------------Libraries------------------------------------------
#include "BasicStepperDriver.h" // Library for the drive stepper motor bi-polar
#include <HCSR04.h>             // Library for the distance sensor code. 
//---------------PINS------------------------------------------
#define TXD 1               // = pin 31 - PD1
#define RXD 0               // = pin 30 - PD0

#define MOTORSLEEP 8        // = pin 12 - PB0
#define MOTORSTEP 9         // = pin 13 - PB1
#define MOTORDIR 10         // = pin 14 - PB2

#define DISTSENSORTRIG 2    // = pin 32 - PD2
#define DISTSENSORECHO 3    // = pin 1 - PD3

#define WATERLEVELSENSOR 4  // = pin 2 - PD4

#define FOUNTAINLIGHT 5     // = pin 9 - PD5

#define ONBOARDLED 7        // = pin 11 - PD7

#define LIGHTSENSOR 14      // = pin 23 - PC0 - A0

#define ONBOARDSWITCH 14    // = pin 24 - PC1 - A1

//---------------CONSTANT------------------------------------------

#define MICROSTEPS 8                // Number of microsteps for the stepper motor.
#define STEPS_ONE_REVOLUTION 200    // Number of step of revolution for the motor.
#define RPM 100                     // Number of rotation per minute for the stepper motor. 
#define NUMBERSTEPPERCALL 10        // Number of step that the stepper motor make when we call the function pumpwater()
#define LIGHTINTERVAL 20 // mVolts  // The variation of the light it take to change the light state.
#define LIGHTTHRESHOLD 20           // The light threshold for not enought light state (mVolts).
#define DISTANCETHRESHOLD 15.0      // The distance in front of the fountain that the cat need to be to start the pump.(Cm)

int enoughtLight = 0 ;    // 0 = no , 1 = yes
int quantityOfLight = 0;  // mVolts


BasicStepperDriver StepperMotor(STEPS_ONE_REVOLUTION, MOTORDIR, MOTORSTEP, MOTORSLEEP);// 2-wire basic config, microstepping is hardwired on the driver
UltraSonicDistanceSensor distanceSensor(DISTSENSORTRIG,DISTSENSORECHO);  // Initialize sensor that uses digital pins 13 and 12.


void setup() {
  Serial.begin(9600);  // We initialize serial connection so that we could print values from sensor.
  // initialize digital pin  
  pinMode(WATERLEVELSENSOR, INPUT); // initialize digital as an input.
  pinMode(FOUNTAINLIGHT, OUTPUT);   // initialize digital as an output.
  pinMode(ONBOARDLED, OUTPUT);      // initialize digital as an output.
  pinMode(LIGHTSENSOR, INPUT);      // initialize digital as an input.
  pinMode(ONBOARDSWITCH, INPUT);    // initialize digital as an input.
  
  StepperMotor.begin(RPM, MICROSTEPS);    // Create and set the stepper motor.
  StepperMotor.setEnableActiveState(LOW); // Activate the stepper motor.

  digitalWrite(ONBOARDLED, HIGH);         //Close the onboard LED.
}
// Function of active the stepper motor to pump water in the fountain.
void PumpWater()
{
  StepperMotor.enable(); // Enable the stepper motor
  StepperMotor.move(NUMBERSTEPPERCALL*MICROSTEPS); // Turn the number of steps that is define in constant section.
  StepperMotor.disable(); // Disable the stepper motor to cut the current in the drive.
  delay(1); // Wait 1ms to give the stepper motor time to make one step.
}
// Fucntion to turn the fountain light ON.
void TurnLightOn()
{
  digitalWrite(FOUNTAINLIGHT, HIGH); // Set logic level HIGH to the pin.
}
// Fucntion to turn the fountain light OFF.
void TurnLightOff()
{
  digitalWrite(FOUNTAINLIGHT, LOW); // Set logic level LOW to the pin.
}
// Function to detect is it have enought light in the room. 
int isEnoughtLight()
{
  int Lvalue = analogRead(LIGHTSENSOR);       // declare and assign read light
  int mVolt = map (Lvalue,0, 1023, 0, 5000);  // Convert the value from the sensor in mVolts.
  
  if( abs(quantityOfLight - mVolt) > LIGHTINTERVAL) // If the quantity of light of the sensor change more that the light interval
  {
    if(mVolt >= LIGHTTHRESHOLD) // if the room it's birght
    {
      return 1; // Return the stat enough light.
    }
    else if(mVolt < LIGHTTHRESHOLD) // if the room it's dark.
    {
      return 0; // Return the stat NOT enough light.
    }
    quantityOfLight = mVolt;
  }
  else // else keep the same state and return the answer.
  {
    if(quantityOfLight >= LIGHTTHRESHOLD) // if the room it's birght
    {
      return 1; // Return the stat enough light
    }
    else if(quantityOfLight < LIGHTTHRESHOLD) // if the room it's dark.
    {
      return 0; // Return the stat NOT enough light.
    }  
  }
}

// Function that check if it have enought water in the fountain.
int isEnoughtWater()
{
  return(!digitalRead(WATERLEVELSENSOR)); // Read the pin state of the sensor and return the answer.
}

// Function that make blinking the light fountain every 500 ms. 
void blinkFountainLight()
{
    TurnLightOn();
    delay(500);
    TurnLightOff();
    delay(500);  
}
// Function that check if the cat is in front of the fountain.
int isNear()
{
    float distance = distanceSensor.measureDistanceCm(); // Read the distance from the sensor in centimeters.
    if(distance <= DISTANCETHRESHOLD) // if the distance cat is under the threshold.
    {
      return 1;  // Return the answer is near.
    }
    else 
    {
      return 0;  // Return the answer is far.
    }
  
}

void loop() {

  if (isEnoughtWater()) // if it have enought water in the fountain.
  {
     if(!isEnoughtLight()) // if it's don't have enought light in the room.
     {
        TurnLightOn(); // turn the foutain light ON.
        if(isNear())   // if the cat is in front of the fountain.
        {
          PumpWater(); // pump the water.
        }
     }
     else
     {
        TurnLightOff();  // turn the foutain light OFF.
        if(isNear())     // if the cat is in front of the fountain.
        {
          PumpWater();   // pump the water. 
        }
     }
  }
  else 
  {
    blinkFountainLight(); // Blink the fountain light
  }
}
